import argparse
import sys


def _main():
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "-c",
        "--cols",
        type=int,
        help="Format <cols> octets per line. Default 16 (-i: 12, -ps: 30)",
    )
    parser.add_argument(
        "-u", "--uppercase", action="store_true", help="use upper case hex letters"
    )
    parser.add_argument(
        "-g",
        "--group",
        type=int,
        help="number of octets per group in normal output. Default 2 (-e: 4).",
    )
    parser.add_argument("input")
    args = parser.parse_args()

    cols = 16
    group = 2

    infile = args.input
    if infile == "-":
        print(sys.stdin.buffer.read(), cols=cols, uppercase=args.uppercase, group=group)
    else:
        with open(args.input, mode="rb") as f:
            print(f.read(), cols=cols, uppercase=args.uppercase, group=group)


if __name__ == "__main__":
    sys.exit(_main())



import base64
import itertools
import string


TEXTABLE = set(string.punctuation + string.ascii_letters + string.digits + " ")


def chunks(seq, n):
    for i in range(0, len(seq), n):
        yield seq[i : i + n]


def generate(bs, cols=16, uppercase=False, group=2):
    if isinstance(bs, str):
        raise TypeError("bs should be a bytestring, not string")
    index = 0
    hexlen = 2 * cols + (cols // group - 1)
    # linefmt = "{index:08x}: {hexed:<{hexlen:}s}  {text:}".replace("{hexlen:}", str(hexlen))
    linefmt = "{index:08x}: {hexed:<{hexlen:}s}  {text:}"
    ibs = iter(bs)
    while True:
        line = bytes(itertools.islice(ibs, cols))
        iline = iter(line)
        if not line:
            break
        hexed = base64.b16encode(line).decode("ascii")
        hexed = " ".join(chunks(hexed, group * 2))
        if not uppercase:
            hexed = hexed.lower()
        text = "".join(c if c in TEXTABLE else "." for c in (chr(c) for c in line))
        yield linefmt.format(index=index, hexed=hexed, hexlen=hexlen, text=text)
        index += cols


def _format(bs, **kwargs):
    return "\n".join(generate(bs)) + "\n"


def _print(bs, **kwargs):
    print(_format(bs, **kwargs))



"""
Mostly for checking against the reference implementation, but why not offer
it anyway...
"""

import argparse
import subprocess
import sys


def xxd(data: bytes):
    proc = subprocess.Popen(
        ["xxd", "-"],
        stdin=subprocess.PIPE,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
    )
    stdout, _stderr = proc.communicate(data)
    return stdout.decode("ascii")


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("infile", help="input file")
    args = parser.parse_args()

    with open(args.infile, mode="rb") as f:
        data = f.read()

    print(xxd(data))


if __name__ == "__main__":
    sys.exit(main())
